<?php
defined( 'ABSPATH' ) || exit;

if ( !class_exists( 'DPMainClass' ) ) {
    /**
     * Init Class DPMainClass
     */
    class DPMainClass {

        /**
         * Menu ID
         *
         * @var string
         */
        public $menu_id = 'dynamic-plugins';

        /**
         * Menu title
         *
         * @var string
         */
        public $menu_title = 'Installed Plugins';

        /**
         * Menu name
         *
         * @var string
         */
        public $menu_name = 'Dynamic Plugins';


        /**
         * Constructor
         */
        public function __construct() {
            add_action( 'admin_bar_menu', array( $this, 'dyn__admin_bar_item' ), 500 ); // Add node to admin bar
            add_action( 'admin_enqueue_scripts', array( $this, 'dyn__enqueue_back' ) ); // Enqueue Back-End
            add_action( 'wp_enqueue_scripts', array( $this, 'dyn__enqueue_front' ) ); // Enqueue Front-End
            add_action( 'wp_ajax_show_plugins', array( $this, 'dyn__show_plugins' ), 0 ); // Show plugins
            add_action( 'wp_ajax_nopriv_show_plugins', array( $this, 'dyn__show_plugins' ) ); // Show Plugins NoPriv
            add_action( 'wp_ajax_toggle_plugins', array( $this, 'dyn__toggle_plugins' ), 0 ); // Toggle Plugins
        }

        /**
         * Get plugin filename
         */
        public function dyn__plugin_filename( $plugin_name ) {

            // Prevent non authorized user to make action
            if ( !is_user_logged_in() && ( !current_user_can( 'edit_plugins' ) && !current_user_can( 'activate_plugins' ) ) ) :
                return;
            endif;

            require_once ABSPATH . '/wp-admin/includes/plugin.php';
            $plugins = get_plugins();

            // Return if no plugins found
            if ( !$plugins ) :
                return;
            endif;

            foreach ( $plugins as $plugin_file => $plugin_infos ) :
                if ( $plugin_infos['Name'] === $plugin_name ) :
                    return $plugin_file;
                endif;
            endforeach;

            return null;
        }

        /**
         * Add Node to admin bar
         */
        public function dyn__admin_bar_item( WP_Admin_Bar $admin_bar ) {

            if ( !is_user_logged_in() && ( !current_user_can( 'edit_plugins' ) && !current_user_can( 'activate_plugins' ) ) ) :
                return;
            endif;
            $admin_bar->add_menu(
                array(
                    'id'     => $this->menu_id,
                    'parent' => null,
                    'group'  => null,
                    'title'  => $this->menu_title,
                    'meta'   => array(
                        'title' => __( $this->menu_name, $this->menu_id ), //phpcs:disable
                    ),
                )
            );
        }

        /**
         * Enqueue Back-End
         */
        public function dyn__enqueue_back() {

            if ( !is_user_logged_in() && ( !current_user_can( 'edit_plugins' ) && !current_user_can( 'activate_plugins' ) ) ) :
                return;
            endif;

            // JS
            wp_enqueue_script( 'dynamic-plugins-script', DP_ASSETS_URL . 'js/main.js', array( 'jquery' ), 1.0, false );

            // AJAX
            wp_localize_script(
                'dynamic-plugins-script',
                'admin',
                array(
                    'ajax_url' => admin_url( 'admin-ajax.php' ),
                    'nonce'    => wp_create_nonce( 'ajax-nonce' ),
                )
            );

            // CSS
            wp_enqueue_style( 'dynamic-plugins-css', DP_ASSETS_URL . 'css/main.css', array(), 1.0, 'all' );

        }

        /**
         * Enqueue Front-End
         */
        public function dyn__enqueue_front() {

            // Return if user is not loggedin
            if ( !is_user_logged_in() && ( !current_user_can( 'edit_plugins' ) && !current_user_can( 'activate_plugins' ) ) ) :
                return;
            endif;

            // JS
            wp_enqueue_script( 'dynamic-plugins-script', DP_ASSETS_URL . 'js/main.js', array( 'jquery' ), 1.0, false );

            // AJAX
            wp_localize_script( 'dynamic-plugins-script', 'admin', array( 'ajax_url' => admin_url( 'admin-ajax.php' ) ) );

            // CSS
            wp_enqueue_style( 'dynamic-plugins-css', DP_ASSETS_URL . 'css/main.css', array(), 1.0, 'all' );

        }

        /**
         * List Installed Plugins
         */
        public function dyn__show_plugins() {

            // Prevent non authorized user to make action
            if ( !is_user_logged_in() && ( !current_user_can( 'edit_plugins' ) && !current_user_can( 'activate_plugins' ) ) ) :
                return;
            endif;

            require_once ABSPATH . 'wp-admin/includes/plugin.php';
            $active_plugins_infos = array();
            $active_plugins_list  = get_option( 'active_plugins' );
            $active_plugins_array = get_plugins();

            if ( !$active_plugins_array ) :
                wp_send_json_error( 'no-plugins-found' );
            endif;

            foreach ( $active_plugins_array as $active_plugins_array_data ) :

                $plugin_name = $active_plugins_array_data['Name'];
                $plugin_slug = $active_plugins_array_data['TextDomain'];

                // Skip plugin with no slug
                if ( !$plugin_slug ) :
                    continue;
                endif;

                // Skip this plugin
                if ( $plugin_name === 'Dynamic Plugins' ) :
                    continue;
                endif;

                $custom_plugin_path = $this->dyn__plugin_filename( $plugin_name );

                // Is plugin active ?
                if ( in_array( $custom_plugin_path, $active_plugins_list, true ) ) :
                    $active_plugins_infos[] = array(
                        'name'   => $plugin_name,
                        'status' => 'active',
                        'nonce'  => wp_create_nonce( 'activation-nonce' ),
                        'slug'   => $custom_plugin_path,
                    );
                else :
                    $active_plugins_infos[] = array(
                        'name'   => $plugin_name,
                        'status' => 'inactive',
                        'nonce'  => wp_create_nonce( 'activation-nonce' ),
                        'slug'   => $custom_plugin_path,
                    );
                endif;
            endforeach;
            wp_send_json_success( $active_plugins_infos );

        }

        /**
         * Enable / Disable Plugins
         */
        public function dyn__toggle_plugins() {

            // Prevent non authorized user to make action
            if ( !is_user_logged_in() && ( !current_user_can( 'edit_plugins' ) && !current_user_can( 'activate_plugins' ) ) ) :
                return;
            endif;

            if ( !wp_verify_nonce( $_POST['nonce'], 'activation-nonce' ) ) { // phpcs:ignore
                exit( 'Dont play with me Bro ! 😈' );
            }

            if ( isset( $_POST['status'] ) && !empty( $_POST['status'] ) ) : // phpcs:ignore
                $plugin_status = $_POST['status']; // phpcs:ignore
            endif;

            if ( isset( $_POST['slug'] ) && !empty( $_POST['slug'] ) ) : // phpcs:ignore
                $plugin_slug = $_POST['slug']; // phpcs:ignore
            endif;

            switch ( $plugin_status ) :
                case 'active':
                    deactivate_plugins( $plugin_slug );
                    wp_send_json_success( 'deactivated' );
                    break;
                case 'inactive':
                    activate_plugins( $plugin_slug );
                    wp_send_json_success( 'activated' );
                    break;
            endswitch;

        }
    }
}

new DPMainClass();
